import ij.IJ;
import ij.ImagePlus;
import ij.WindowManager;
import ij.gui.GenericDialog;
import ij.plugin.PlugIn;
import ij.process.ImageProcessor;
import imagescience.images.Axes;
import imagescience.images.ColorImage;
import imagescience.images.Dimensions;
import imagescience.images.Image;
import imagescience.transforms.Mirror;
import imagescience.utilities.I5DResource;
import imagescience.utilities.Messenger;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Panel;
import java.awt.image.ColorModel;

public class TJ_Mirror implements PlugIn {

    private static boolean x = false;
    private static boolean y = false;
    private static boolean z = false;
    private static boolean t = false;
    private static boolean c = false;

    public void run(String arg) {

	msg.debug(TJ_Options.debug);
	msg.debug("TransformJ "+TJ_Options.VERSION+": Mirror");

	if (IJ.versionLessThan("1.35i")) return;

	final ImagePlus imp = WindowManager.getCurrentImage();
	if (imp == null)  { error("There are no images open"); return; }

	GenericDialog gd = new GenericDialog("TransformJ: Mirror");
	gd.addCheckbox(" x-mirror input image    ",x);
	gd.addCheckbox(" y-mirror input image    ",y);
	gd.addCheckbox(" z-mirror input image    ",z);
	gd.addCheckbox(" t-mirror input image    ",t);
	gd.addCheckbox(" ch-mirror input image    ",c);
	gd.addPanel(new Panel(),GridBagConstraints.EAST,new Insets(0,0,0,0));
	gd.showDialog();

	if (gd.wasCanceled()) return;

	x = gd.getNextBoolean();
	y = gd.getNextBoolean();
	z = gd.getNextBoolean();
	t = gd.getNextBoolean();
	c = gd.getNextBoolean();

	try {
	    final Image img = Image.wrap(imp);
	    final Image newimg = img.duplicate();
	    final Mirror mrr = new Mirror();
	    mrr.debug(TJ_Options.debug);
	    mrr.run(newimg,new Axes(x,y,z,t,c));
	    ImagePlus newimp = newimg.imageplus();
	    newimp.setCalibration(imp.getCalibration().copy());
	    final ImageProcessor ip = imp.getProcessor();
	    final ImageProcessor newip = newimp.getProcessor();
	    final ColorModel cm = ip.getColorModel();
	    newip.setColorModel(cm);
	    newimp.getStack().setColorModel(cm);
	    newip.setMinAndMax(ip.getMin(),ip.getMax());
	    boolean i5dinstalled = false;
	    try { Class.forName("Image5D"); i5dinstalled = true; } catch (Exception e) { }
	    if (i5dinstalled && !(newimg instanceof ColorImage)) {
		final Dimensions newdims = newimg.dimensions();
		if (newdims.t > 1 || newdims.s > 1) {
		    newimp = I5DResource.convert(newimp,true);
		    if (I5DResource.instance(imp)) {
			final int[][] idx = new int[2][newdims.s];
			if (c) for (int i=0; i<newdims.s; ++i) {
			    idx[0][i] = i + 1;
			    idx[1][i] = newdims.s - i;
			} else for (int i=0; i<newdims.s; ++i)
			    idx[0][i] = idx[1][i] = i + 1;
			I5DResource.transfer(imp,newimp,idx);
		    }
		}
	    }
	    newimp.changes = TJ_Options.asksave;
	    newimp.show();

	} catch (OutOfMemoryError e) {
	    error("Not enough memory for this operation");
	}
    }

    private void error(final String message) { msg.error("TransformJ: Error", message+"."); }

    private final Messenger msg = new Messenger();

}
